package abc.ar;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import abc.ar.weaving.weaver.around.ARWeaver;
import abc.aspectj.parse.AbcLexer;
import abc.aspectj.parse.LexerAction_c;
import abc.main.Debug;
import abc.weaving.weaver.Weaver;

/* Aspect refinement extension
 * 
 * @author Christian Kaestner and Sven Apel
 */

public class AbcExtension extends abc.main.AbcExtension {

	private static AbcExtension instance = null;

	public static AbcExtension v() {
		return instance;
	}

	public AbcExtension() {
		instance = this;
	}

	private List _aspectRefinements = new ArrayList();

	protected void collectVersions(StringBuffer versions) {
		super.collectVersions(versions);
		versions.append(" with aspect refinement (ar) extension " + new abc.ar.Version().toString() + "\n");
	}

	public abc.aspectj.ExtensionInfo makeExtensionInfo(Collection jar_classes, Collection aspect_sources) {

		try {
			findEquationFiles(aspect_sources);
			printAspectSources(aspect_sources);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return new abc.ar.ExtensionInfo(jar_classes, aspect_sources);
	}

	private void printAspectSources(Collection aspect_sources) {
		System.out.println("Compiler Input:");
		for (Iterator iter = aspect_sources.iterator(); iter.hasNext();) {
			System.out.println("* " + iter.next());
		}
	}

	private void findEquationFiles(Collection aspect_sources) throws IOException {
		Collection equationFiles = new ArrayList();
		for (Iterator iter = aspect_sources.iterator(); iter.hasNext();) {
			String filename = (String) iter.next();
			int p = filename.lastIndexOf('.');
			String extention = "";
			if (p >= 0)
				extention = filename.substring(p);
			if (extention.equals(".equation")) {
				equationFiles.add(filename);
				iter.remove();
			}
		}

		for (Iterator i = equationFiles.iterator(); i.hasNext();) {
			loadEquationFile((String) i.next(), _aspectRefinements);
		}
		aspect_sources.addAll(_aspectRefinements);
	}

	private void loadEquationFile(String arfilename, Collection aspect_sources) throws IOException {
		System.out.println("Loading Aspect Refinement file " + arfilename);

		List featureOrder = new ArrayList();

		File file = new File(arfilename);
		if (!file.exists())
			new IOException("Equation File " + arfilename + " does not exist.");
		BufferedReader fileReader = new BufferedReader(new FileReader(file));
		String line;
		while ((line = fileReader.readLine()) != null) {
			if (line.startsWith("//") || line.startsWith(";")) continue;
			if (Debug.v().refinementFrontentVisitor)
				System.out.println(line);
			findSourcesInDir(line, aspect_sources);
			featureOrder.add(line);// TODO featurename aus pfad extrahieren
									// falls ntig.
		}
		getRefinementManager().addFeatureOrder(featureOrder);
	}

	int dummyFeatureCounter = 0;

	/**
	 * returns temporary feature names to distinguish between different aspects
	 * with the same name during parsing
	 */
	public String getFeatureNameDummy() {
		return "temp_feature" + (++dummyFeatureCounter);
	}

	/**
	 * copied from abc.Main
	 * 
	 * @param dir
	 * @param sources
	 * @throws IllegalArgumentException
	 */
	private void findSourcesInDir(String dir, Collection sources) throws IllegalArgumentException {
		File file = new File(dir);
		File[] files = file.listFiles();
		if (files == null) {
			throw new IllegalArgumentException("Sourceroot " + dir + " is not a directory");
		}
		for (int i = 0; i < files.length; i++) {
			if (files[i].isDirectory())
				findSourcesInDir(files[i].getAbsolutePath(), sources);
			else {
				String fileName = files[i].getPath();
				if (fileName.endsWith(".java") || fileName.endsWith(".aj")) {
					sources.add(fileName);
				}
			}
		}
	}

	protected List/* <ShadowType> */listShadowTypes() {
		List/* <ShadowType */shadowTypes = super.listShadowTypes();

		// shadowTypes.add(CastShadowMatch.shadowType());
		// shadowTypes.add(ThrowShadowMatch.shadowType());

		return shadowTypes;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see abc.main.AbcExtension#initLexerKeywords(abc.aspectj.parse.AbcLexer)
	 */
	public void initLexerKeywords(AbcLexer lexer) {
		// Add the base keywords
		super.initLexerKeywords(lexer);

		// keyword for the "global pointcut" extension
		lexer.addGlobalKeyword("refines", new LexerAction_c(new Integer(abc.ar.parse.sym.REFINES), new Integer(lexer
				.aspectj_state())));
	}

	private RefinementManager _refinementList = null;

	public RefinementManager getRefinementManager() {
		if (_refinementList == null)
			_refinementList = new RefinementManager();
		return _refinementList;
	}
	
	public void addJimplePacks() {
		
	}
	
    private ARWeaver weaver;

	public Weaver getWeaver() {
		if (weaver == null)
			weaver = new ARWeaver();

		return weaver;
	}
}
