package abc.ar.ast;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import polyglot.ast.Expr;
import polyglot.ast.Node;
import polyglot.ast.Receiver;
import polyglot.ast.Special;
import polyglot.types.MethodInstance;
import polyglot.types.SemanticException;
import polyglot.util.Position;
import polyglot.visit.TypeChecker;
import abc.ar.types.ARContext;
import abc.aspectj.ast.AJNodeFactory;
import abc.aspectj.ast.ProceedCall;
import abc.aspectj.extension.AJCall_c;

public class ARCall_c extends AJCall_c {

	public ARCall_c(Position pos, Receiver target, String name, List arguments) {
		super(pos, target, name, arguments);
	}

	public Node typeCheck(TypeChecker tc) throws SemanticException {
		ARContext context = (ARContext) tc.context();
		// check if call is made to super.adviceName(), that can be transformed
		// to a proceed call
		if (context.inAdvice() && context.currentNamedAdvice() instanceof NamedAdviceDecl
				&& ((NamedAdviceDecl) context.currentNamedAdvice()).getSpec() instanceof NamedAround) {
			//is super call?
			if ((this.target instanceof Special) && ((Special)this.target).kind() == Special.SUPER) {
				//has the same name as the advice?
				NamedAround namedAround = (NamedAround) context.currentNamedAdvice().getSpec();
				if (this.name.equals(namedAround.name())) {
					//has the same formals as the required proceed call?
					MethodInstance pi = context.proceedInstance();
					List argTypes = new ArrayList(arguments.size());
					for (Iterator i = arguments.iterator(); i.hasNext(); ) {
						 Expr e = (Expr) i.next();
						 argTypes.add(e.type());
					}
					if (pi.callValid(argTypes)){
						//replace the super-call with a proceed call
						ProceedCall proceedCall = ((AJNodeFactory)tc.nodeFactory()).ProceedCall(this.position(),this.target(), this.arguments());
						return proceedCall.typeCheck(tc);
					}
				}
			}
		}
		return super.typeCheck(tc);
	}
}
