/* abc - The AspectBench Compiler
 * Copyright (C) 2004 Oege de Moor
 * Copyright (C) 2004 Aske Simon Christensen
 *
 * This compiler is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This compiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this compiler, in the file LESSER-GPL;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package abc.ar.ast;

import java.util.Iterator;
import java.util.List;

import polyglot.ast.Formal;
import polyglot.ast.MethodDecl;
import polyglot.ast.TypeNode;
import polyglot.util.CodeWriter;
import polyglot.util.Position;
import polyglot.visit.PrettyPrinter;
import abc.aspectj.ast.Around_c;
import abc.weaving.aspectinfo.AbcFactory;

/**
 * specification part of around advice.
 * 
 * @author Oege de Moor
 * @author Aske Simon Christensen
 */
public class NamedAround_c extends Around_c implements NamedAround {
	private MethodDecl namedAround;

	private String name;

	public String name() {
		return name;
	}

	public NamedAround_c(Position pos, String name, TypeNode returnType, List formals) {
		super(pos, returnType, formals);
		this.name = name;
	}

	public String kind() {
		return "named_" + super.kind();
	}

	public String toString() {
		return "named_" + super.toString();
	}

	public void prettyPrint(CodeWriter w, PrettyPrinter tr) {
		w.write("named_");
		super.prettyPrint(w, tr);
	}

	public void setNamedAround(MethodDecl namedAround) {
		this.namedAround=namedAround;
	}

	public MethodDecl namedAround() {
		return namedAround;
	}

    public abc.weaving.aspectinfo.AdviceSpec makeAIAdviceSpec() {
    	abc.weaving.aspectinfo.AbcType rtype = AbcFactory.AbcType(returnType.type());
    	abc.weaving.aspectinfo.MethodSig psig = AbcFactory.MethodSig(proceed());
//    	abc.weaving.aspectinfo.MethodSig nasig = AbcFactory.MethodSig(namedAround());
    	abc.weaving.aspectinfo.MethodSig nasig=null;
    	return new abc.ar.weaving.aspectinfo.NamedAroundAdvice(rtype, psig, nasig, position());
    }

	
}
