package abc.ar.visit;

import polyglot.ast.Node;
import polyglot.ast.SourceFile;
import polyglot.frontend.Job;
import polyglot.types.SemanticException;
import polyglot.types.TypeSystem;
import polyglot.visit.ContextVisitor;
import polyglot.visit.NodeVisitor;
import abc.ar.ast.ARNodeFactory;
import abc.ar.ast.RefinableAspectDecl;

/**
 * this visitor collects the feature name by extracting the last part of the
 * path for all aspects.
 * 
 * afterwards each aspect is identifiable by name and featurename.
 * 
 * @author Christian Kaestner and Sven Apel
 */
public class FeatureNameResolverVisitor extends ContextVisitor {

	public FeatureNameResolverVisitor(Job job, TypeSystem ts, ARNodeFactory nf) {
		super(job, ts, nf);
	}

	private SourceFile _sourceFile = null;
	private static int counter;

	protected NodeVisitor enterCall(Node parent, Node n) throws SemanticException {
		if (n instanceof SourceFile)
		    _sourceFile = (SourceFile) n;
		return super.enterCall(parent, n);
	}

	protected Node leaveCall(Node n) throws SemanticException {
		if (n instanceof SourceFile)
			_sourceFile = null;

		if (n instanceof RefinableAspectDecl) {
			RefinableAspectDecl a = (RefinableAspectDecl) n;
			
 			a.setFeatureName(getCurrentFeatureName());
		}

		return super.leaveCall(n);
	}


	private String getCurrentFeatureName() throws SemanticException {

		if (_sourceFile==null)
			throw new SemanticException("Aspect not defined in a source file");

				
		String path = _sourceFile.source().path();
		return getFeatureNameFromPath(path);
	}


	private String getFeatureNameFromPath(String filename) {

		filename = filename.replace('\\', '/');
		int p = filename.lastIndexOf('/');
		String filepart = filename.substring(p + 1);
		String pathpart = filename.substring(0, (p > 0 ? p : 0));

		p = filepart.indexOf('.');
		filepart = filepart.substring(0, p);

		p = pathpart.lastIndexOf('/');
		if (p >= 0)
			pathpart = pathpart.substring(p + 1);

		if (pathpart.equals(""))
			pathpart = "unknown" + (++counter);

		return pathpart;
	}

}
