package abc.ar.visit;

import polyglot.ast.AmbReceiver;
import polyglot.ast.Node;
import polyglot.ast.SourceFile;
import polyglot.frontend.Job;
import polyglot.types.SemanticException;
import polyglot.types.TypeSystem;
import polyglot.visit.ContextVisitor;
import polyglot.visit.NodeVisitor;
import abc.ar.AbcExtension;
import abc.ar.ast.ARNodeFactory;
import abc.ar.ast.AspectRefinementDecl;
import abc.ar.ast.PCSuperName;
import abc.ar.ast.RefinableAspectDecl;
import abc.main.Debug;

/**
 * 
 * @author Christian Kaestner and Sven Apel
 */
public class PointcutSuperResolver extends ContextVisitor {

	public PointcutSuperResolver(Job job, TypeSystem ts, ARNodeFactory nf) {
		super(job, ts, nf);
	}

	private AspectRefinementDecl _refinement = null;

	protected NodeVisitor enterCall(Node parent, Node n) throws SemanticException {
		if (n instanceof AspectRefinementDecl)
			_refinement = (AspectRefinementDecl) n;
		return super.enterCall(parent, n);
	}

	protected Node leaveCall(Node n) throws SemanticException {
		if (n instanceof AspectRefinementDecl)
			_refinement = null;

		if (n instanceof PCSuperName) {
			PCSuperName pc = (PCSuperName) n;
			
			if (Debug.v().refinementFrontentVisitor)
				System.out.println("found super pointcut: "+pc);
			
			if (_refinement==null)
				throw new SemanticException("Super keyword used for a pointcut outside a refinement.");
			
			if (pc.target()!=null)
				throw new SemanticException("Super keyword used in combination with target.");

			
			String superFeature=AbcExtension.v().getRefinementManager().getSuperFeature(_refinement.getOriginalName(), _refinement.getFeatureName());
			String superAspectName=AbcExtension.v().getRefinementManager().getNameForAspect(_refinement.getOriginalName(), superFeature);
			
			AmbReceiver x = nf.AmbReceiver(pc.position(), superAspectName);
			n=pc.target(x);
		}

		return super.leaveCall(n);
	}



}
