package abc.ar.visit;

import polyglot.ast.Node;
import polyglot.frontend.Job;
import polyglot.types.SemanticException;
import polyglot.types.TypeSystem;
import polyglot.visit.ContextVisitor;
import abc.ar.AbcExtension;
import abc.ar.ast.ARNodeFactory;
import abc.ar.ast.RefinableAspectDecl;
import abc.main.Debug;

/**
 * this class renames all aspects that are affected by refinements
 * 
 * @author Christian Kaestner and Sven Apel
 */
public class RefinedAspectRenamingVisitor extends ContextVisitor {

	public RefinedAspectRenamingVisitor(Job job, TypeSystem ts, ARNodeFactory nf) {
		super(job, ts, nf);
	}

	protected Node leaveCall(Node n) throws SemanticException {
		if (n instanceof RefinableAspectDecl) {
			RefinableAspectDecl a = (RefinableAspectDecl) n;

			String newName = AbcExtension.v().getRefinementManager().getNameForAspect(a.getOriginalName(),
					a.getFeatureName());

			String oldName = a.name();

			if (!oldName.equals(newName)) {
				// rename type
				a.type().name(newName);
				ts.parsedResolver().addNamed(a.type());
				// rename
				n = a.name(newName);
				
				if (Debug.v.refinementFrontentVisitor)
					System.out.println(">> renamed aspect " + oldName + " to " + newName);
			}
		}

		return super.leaveCall(n);
	}

}
