package abc.ar.visit;

import polyglot.ast.AmbTypeNode;
import polyglot.ast.Node;
import polyglot.frontend.Job;
import polyglot.types.SemanticException;
import polyglot.types.Type;
import polyglot.types.TypeSystem;
import polyglot.visit.ContextVisitor;
import abc.ar.AbcExtension;
import abc.ar.RefinementManager;
import abc.ar.ast.ARNodeFactory;
import abc.ar.ast.AspectRefinementDecl;
import abc.ar.ast.RefinableAspectDecl;

/**
 * pass that accually applies the new inheritance to the refinements and refined aspects.
 * 
 * refined aspects are marked abstract and refinements are declared with their new parent.
 *  
 * @author Christian Kaestner and Sven Apel
 */
public class RefinementHierachieWeaver extends ContextVisitor {

	private final RefinementManager refinementList;

	public RefinementHierachieWeaver(Job job, TypeSystem ts, ARNodeFactory nf) {
		super(job, ts, nf);
		refinementList=AbcExtension.v().getRefinementManager();
	}

	protected Node leaveCall(Node n) throws SemanticException {
		// refined aspects are marked abstract
		if (n instanceof RefinableAspectDecl) {
			RefinableAspectDecl aspect = (RefinableAspectDecl) n;
			if (AbcExtension.v().getRefinementManager().isRefined(aspect)) {
				aspect.type().flags(aspect.type().flags().Abstract());
				n = aspect.flags(aspect.flags().Abstract());
			}
		}

		// aspect refinements are declared with a new parent
		if (n instanceof AspectRefinementDecl) {
			AspectRefinementDecl refinement = ((AspectRefinementDecl) n);

			// asure no super class set already
			if (refinement.superClass() != null)
				throw new SemanticException("> Aspect refinement: " + refinement.getOriginalName()
						+ " already has a super class.");

			//set new super class
			String superFeature=refinementList.getSuperFeature(refinement.getOriginalName(),refinement.getFeatureName());
			assert superFeature!=null;
			String newSuperClassName=refinementList.getNameForAspect(refinement.getOriginalName(), superFeature);
			Type newSuperType=ts.typeForName(newSuperClassName);//refinement.getRefinedAspect().type();
			
			// type
			refinement.type().superType(newSuperType);

			// ast node
			AmbTypeNode tn = nf.AmbTypeNode(refinement.position(), newSuperClassName);
			n = refinement.superClass(tn);
		}

		return super.leaveCall(n);
	}

}
